@@ -24,7 +24,7 @@ from email.header import decode_header
 
 log = logging.getLogger('bugzilla')
 
-version = '0.6.2'
+version = '0.7.0'
 user_agent = 'Python-urllib2/%s bugzilla.py/%s' % \
         (urllib2.__version__,version)
 
@@ -253,6 +253,12 @@ class BugzillaBase(object):
         If 'user' and 'password' are both set, we'll run login(). Otherwise
         you'll have to login() yourself before some methods will work.
         '''
+        if not '://' in url:
+            log.debug('No scheme given for url, assuming https')
+            url = 'https://' + url
+        if url.count('/') < 3:
+            log.debug('No path given for url, assuming /xmlrpc.cgi')
+            url = url + '/xmlrpc.cgi'
         # Set up the transport
         if url.startswith('https'):
             self._transport = SafeCookieTransport()
@@ -342,7 +348,8 @@ class BugzillaBase(object):
         '''Calls getBugFields, which returns a list of fields in each bug
         for this bugzilla instance. This can be used to set the list of attrs
         on the Bug object.'''
-        if force_refresh or not self._bugfields:
+        if force_refresh or self._bugfields is None:
+            log.debug("Refreshing bugfields")
             try:
                 self._bugfields = self._getbugfields()
             except xmlrpclib.Fault, f:
@@ -352,6 +359,8 @@ class BugzillaBase(object):
                 else:
                     # something bad actually happened on the server. blow up.
                     raise f
+            self._bugfields.sort()
+            log.debug("bugfields = %s", self._bugfields)
 
         return self._bugfields
     bugfields = property(fget=lambda self: self.getbugfields(),
@@ -496,9 +505,13 @@ class BugzillaBase(object):
         '''IMPLEMENT ME: Return a list of short bug dicts, one for each of the
         given bug ids'''
         raise NotImplementedError
-    def _query(self,query):
+    def build_query(self, *args, **kwargs):
+        raise NotImplementedError("This version of bugzilla does not "
+                                  "support bug querying.")
+    def _query(self, query):
         '''IMPLEMENT ME: Query bugzilla and return a list of matching bugs.'''
-        raise NotImplementedError
+        raise NotImplementedError("This version of bugzilla does not "
+                                  "support bug querying.")
 
     # these return Bug objects
     def getbug(self,id):
@@ -527,7 +540,9 @@ class BugzillaBase(object):
         Also see the _query() method for details about the underlying
         implementation.
         '''
+        log.debug("Calling query with: %s", query)
         r = self._query(query)
+        log.debug("Query returned %s bugs", len(r['bugs']))
         return [_Bug(bugzilla=self,dict=b) for b in r['bugs']]
 
     def simplequery(self,product,version='',component='',string='',matchtype='allwordssubstr'):
@@ -542,6 +557,16 @@ class BugzillaBase(object):
         q = {'product':product,'version':version,'component':component,
              'long_desc':string,'long_desc_type':matchtype}
         return self.query(q)
+    def pre_translation(self, query):
+        '''In order to keep the API the same, Bugzilla4 needs to process the
+        query and the result. This also applies to the refresh() function
+        '''
+        pass
+    def post_translation(self, query, bug):
+        '''In order to keep the API the same, Bugzilla4 needs to process the
+        query and the result. This also applies to the refresh() function
+        '''
+        pass
 
     #---- Methods for modifying existing bugs.
 
@@ -695,15 +720,19 @@ class BugzillaBase(object):
     createbug_required = ('product','component','version','short_desc','comment',
                           'rep_platform','bug_severity','op_sys','bug_file_loc')
 
-    # List of field aliases. If a createbug() call lacks a required field, but
-    # a corresponding alias field is present, we'll automatically switch the
-    # field name. This lets us avoid having to change the call to match the
-    # bugzilla instance quite so much.
-    field_aliases = (('summary','short_desc'),
-                     ('description','comment'),
-                     ('platform','rep_platform'),
-                     ('severity','bug_severity'),
-                     ('status','bug_status'))
+    # List of field aliases. Maps old style RHBZ parameter names to actual
+    # upstream values. Used for createbug() and query include_fields at
+    # least.
+    #
+    # Format is (currentname, oldname)
+    field_aliases = (('summary', 'short_desc'),
+                     ('description', 'comment'),
+                     ('platform', 'rep_platform'),
+                     ('severity', 'bug_severity'),
+                     ('status', 'bug_status'),
+                     ('id', 'bug_id'),
+                     ('blocks', 'blockedby'),
+                     ('whiteboard', 'status_whiteboard'))
 
     def _createbug(self,**data):
         '''IMPLEMENT ME: Raw xmlrpc call for createBug()
@@ -1204,21 +1233,31 @@ class _Bug(object):
         return '<Bug #%i on %s at %#x>' % (self.bug_id,self.bugzilla.url,
                                            id(self))
 
-    def __getattr__(self,name):
-        if 'bug_id' in self.__dict__:
-            if self.bugzilla.bugfields and name not in self.bugzilla.bugfields:
-                # We have a list of fields, and you ain't on it.
-                # Check the aliases
-                for a in self.bugzilla.field_aliases:
-                    if a[0] == name: return getattr(self, a[1])
-                    if a[1] == name: return getattr(self, a[0])
-                # Not in the aliases. Bail out.
-                raise AttributeError, "field %s not in bugzilla.bugfields" % name
-            log.debug("Bug %i missing %s - doing refresh()", self.bug_id, name)
-            self.refresh()
+    def __getattr__(self, name):
+        if not ('bug_id' in self.__dict__ or 'id' in self.__dict__):
+            # This is fatal, since we have no ID to pass to refresh()
+            # Can happen if a messed up include_fields is passed to query
+            raise AttributeError("No bug ID cached for bug object")
+
+        refreshed = False
+        while True:
             if name in self.__dict__:
                 return self.__dict__[name]
-        raise AttributeError, "Bug object has no attribute '%s'" % name
+
+            # Check field aliases
+            for newname, oldname in self.bugzilla.field_aliases:
+                if name == newname and oldname in self.__dict__:
+                    return self.__dict__[oldname]
+                if name == oldname and newname in self.__dict__:
+                    return self.__dict__[newname]
+
+            log.debug("Bug %i missing %s - doing refresh()", self.bug_id, name)
+            if refreshed:
+                break
+            self.refresh()
+            refreshed = True
+
+        raise AttributeError("Bug object has no attribute '%s'" % name)
 
     def __getstate__(self):
         sd = self.__dict__
@@ -1237,7 +1276,29 @@ class _Bug(object):
         if self.bugzilla.bugfields:
             for k in self.bugzilla.bugfields:
                 self.__dict__.setdefault(k)
+
+            # Sync alias values. Otherwise we might have just set
+            # 'bug_id' = None, when the __dict__ already had a valid 'id'
+            for newname, oldname in self.bugzilla.field_aliases:
+                def setval(v1, v2):
+                    if v1 is None:
+                        return v2
+                    return v1
+
+                new = self.__dict__.get(newname)
+                old = self.__dict__.get(oldname)
+                self.__dict__[newname] = setval(new, old)
+                self.__dict__[oldname] = setval(old, new)
+
         r = self.bugzilla._getbug(self.bug_id)
+        # faking a query
+        q = {}
+        q['id'] = str(self.bug_id)
+        q['column_list'] = self.bugzilla.bugfields
+
+        # running post translation
+        self.bugzilla.post_translation(q, r)
+
         self.__dict__.update(r)
 
     def reload(self):
@@ -1354,7 +1415,6 @@ class _Bug(object):
     def deletecc(self,cclist,comment=''):
         '''Removes the given email addresses from the CC list for this bug.'''
         self.bugzilla._updatecc(self.bug_id,cclist,'delete',comment)
-
     def get_flag_type(self, name):
         """Return flag_type information for a specific flag"""
 
