@@ -1,6 +1,6 @@
 # rhbugzilla.py - a Python interface to Red Hat Bugzilla using xmlrpclib.
 #
-# Copyright (C) 2008, 2009 Red Hat Inc.
+# Copyright (C) 2008-2012 Red Hat Inc.
 # Author: Will Woods <wwoods@redhat.com>
 #
 # This program is free software; you can redistribute it and/or modify it
@@ -9,39 +9,42 @@
 # option) any later version.  See http://www.gnu.org/copyleft/gpl.html for
 # the full text of the license.
 
+import copy
+import logging
+import xmlrpclib
+
 import bugzilla.base
 from bugzilla.bugzilla3 import Bugzilla3, Bugzilla34
-import copy, xmlrpclib
+from bugzilla.bugzilla4 import Bugzilla4
 
-class RHBugzilla(bugzilla.base.BugzillaBase):
+log = logging.getLogger('bugzilla')
+
+class RHBugzilla(Bugzilla4):
     '''Concrete implementation of the Bugzilla protocol. This one uses the
-    methods provided by Red Hat's Bugzilla 2.18 variant.
+    methods provided by Red Hat's Bugzilla 4.2+ instance, which is a superset
+    of the Bugzilla 4.2 methods. The additional methods (e.g. Bug.update)
+    should make their way into a later upstream Bugzilla release.
 
-    RHBugzilla supports XMLRPC MultiCall. The methods which start with a
-    single underscore are thin wrappers around XMLRPC methods and should thus
-    be safe for multicall use.
+    Note that RHBZ4 *also* supports most of the old RHBZ methods, under the
+    'bugzilla' namespace, so we use those when BZ4 methods aren't available.
 
-    Documentation for most of these methods can be found here:
-    https://bugzilla.redhat.com/docs/en/html/api/extensions/RedHat/lib/WebService/CompatBugzilla.html
+    This class was written using bugzilla.redhat.com's API docs:
+    https://bugzilla.redhat.com/docs/en/html/api/
+
+    By default, _getbugs will multicall getBug(id) multiple times, rather than
+    doing a single Bug.get(idlist) call. You can disable this behavior by
+    setting the 'multicall' property to False. This will make it somewhat
+    faster, but any missing/unreadable bugs will cause the entire call to
+    Fault rather than returning any data.
     '''
 
-    version = '0.3'
-    user_agent = bugzilla.base.user_agent + ' RHBugzilla/%s' % version
+    version = '0.1'
+    user_agent = bugzilla.base.user_agent + ' RHBugzilla4/%s' % version
 
-    def __init__(self,**kwargs):
-        bugzilla.base.BugzillaBase.__init__(self,**kwargs)
+    def __init__(self, **kwargs):
+        Bugzilla4.__init__(self, **kwargs)
         self.user_agent = self.__class__.user_agent
-
-    def _login(self,user,password):
-        '''Backend login method for RHBugzilla.'''
-        return self._proxy.bugzilla.login(user,password)
-
-    def _logout(self):
-        '''Backend logout method for RHBugzilla.'''
-        # "Logouts are not implemented due to the non-session nature of
-        # XML-RPC communication."
-        # That's funny, since we get a (session-based) login cookie...
-        return True
+        self.multicall = kwargs.get('multicall',True)
 
     #---- Methods and properties with basic bugzilla info
 
@@ -80,26 +83,9 @@ class RHBugzilla(bugzilla.base.BugzillaBase):
         return mc
 
     # Connect the backend methods to the XMLRPC methods
-    def _getbugfields(self):
-        return self._proxy.bugzilla.getBugFields()
+
     def _getqueryinfo(self):
         return self._proxy.bugzilla.getQueryInfo()
-    def _getproducts(self):
-        '''Backend _getproducts method for RH Bugzilla. This predates the
-        Bugzilla3 Products stuff, so we need to massage this data to make it
-        fit the proper format'''
-        r = self._proxy.bugzilla.getProdInfo()
-        n = 0
-        prod = []
-        for name,desc in r.iteritems():
-            # We're making up a fake id, since RHBugzilla doesn't use them
-            prod.append({'id':n,'name':name,'description':desc})
-            n += 1
-        return prod
-    def _getcomponents(self,product):
-        if type(product) == int:
-            product = self._product_id_to_name(product)
-        return self._proxy.bugzilla.getProdCompInfo(product)
     def _getcomponentsdetails(self,product):
         if type(product) == int:
             product = self._product_id_to_name(product)
@@ -127,180 +113,12 @@ class RHBugzilla(bugzilla.base.BugzillaBase):
             self._components[product] = r.pop(0)
             self._components_details[product] = r.pop(0)
 
-    #---- Methods for reading bugs and bug info
-
-    def _getbug(self,id):
-        '''Return a dict of full bug info for the given bug id'''
-        return self._proxy.bugzilla.getBug(id)
-    def _getbugsimple(self,id):
-        '''Return a short dict of simple bug info for the given bug id'''
-        r = self._proxy.bugzilla.getBugSimple(id)
-        if r and 'bug_id' not in r:
-            # XXX hurr. getBugSimple doesn't fault if the bug is missing.
-            # Let's synthesize one ourselves.
-            raise xmlrpclib.Fault("Server","Could not load bug %s" % id)
-        else:
-            return r
-    # Multicall methods
-    def _getbugs(self,idlist):
-        '''Like _getbug, but takes a list of ids and returns a corresponding
-        list of bug objects. Uses multicall for awesome speed.'''
-        mc = self._multicall()
-        for id in idlist:
-            mc._getbug(id)
-        raw_results = mc.run()
-        del mc
-        # check results for xmlrpc errors, and replace them with None
-        return bugzilla.base.replace_getbug_errors_with_None(raw_results)
-    def _getbugssimple(self,idlist):
-        '''Like _getbugsimple, but takes a list of ids and returns a
-        corresponding list of bug objects. Uses multicall for awesome speed.'''
-        mc = self._multicall()
-        for id in idlist:
-            mc._getbugsimple(id)
-        raw_results = mc.run()
-        del mc
-        # check results for xmlrpc errors, and replace them with None
-        return bugzilla.base.replace_getbug_errors_with_None(raw_results)
-
-    def _query(self,query):
-        '''Query bugzilla and return a list of matching bugs.
-        query must be a dict with fields like those in in querydata['fields'].
-        Returns a dict like this: {'bugs':buglist,
-                                   'displaycolumns':columnlist,
-                                   'sql':querystring}
-        buglist is a list of dicts describing bugs. You can specify which
-        columns/keys will be listed in the bugs by setting 'column_list' in
-        the query; otherwise the default columns are used (see the list in
-        querydefaults['default_column_list']). The list of columns will be
-        in 'displaycolumns', and the SQL query used by this query will be in
-        'sql'.
-        '''
-        return self._proxy.bugzilla.runQuery(query)
-
     #---- Methods for modifying existing bugs.
 
     # Most of these will probably also be available as Bug methods, e.g.:
     # Bugzilla.setstatus(id,status) ->
     #   Bug.setstatus(status): self.bugzilla.setstatus(self.bug_id,status)
 
-    def _addcomment(self,id,comment,private=False,
-                   timestamp='',worktime='',bz_gid=''):
-        '''Add a comment to the bug with the given ID. Other optional
-        arguments are as follows:
-            private:   if True, mark this comment as private.
-            timestamp: comment timestamp, in the form "YYYY-MM-DD HH:MM:SS"
-            worktime:  amount of time spent on this comment (undoc in upstream)
-            bz_gid:    if present, and the entire bug is *not* already private
-                       to this group ID, this comment will be marked private.
-        '''
-        return self._proxy.bugzilla.addComment(id,comment,self.user,'',
-                private,timestamp,worktime,bz_gid)
-    def _setstatus(self,id,status,comment='',private=False,private_in_it=False,nomail=False):
-        '''Set the status of the bug with the given ID. You may optionally
-        include a comment to be added, and may further choose to mark that
-        comment as private.
-        The status may be anything from querydefaults['bug_status_list'].
-        Common statuses: 'NEW','ASSIGNED','MODIFIED','NEEDINFO'
-        Less common: 'VERIFIED','ON_DEV','ON_QA','REOPENED'
-        'CLOSED' is not valid with this method; use closebug() instead.
-        '''
-        return self._proxy.bugzilla.changeStatus(id,status,self.user,'',
-                comment,private,private_in_it,nomail)
-    def _closebug(self,id,resolution,dupeid,fixedin,comment,isprivate,private_in_it,nomail):
-        '''Raw xmlrpc call for closing bugs. Documentation from Bug.pm is
-        below. Note that we drop the username and password fields because the
-        Bugzilla object contains them already.
-
-        closeBug($bugid, $new_resolution, $username, $password, $dupeid,
-            $new_fixed_in, $comment, $isprivate, $private_in_it, $nomail)
-
-        Close a current Bugzilla bug report with a specific resolution. This will eventually be done in Bugzilla/Bug.pm
-        instead and is meant to only be a quick fix. Please use bugzilla.changesStatus to changed to an opened state.
-        This method will change the bug report's status to CLOSED.
-
-            $bugid
-                # ID of bug report to add comment to.
-            $new_resolution
-                # Valid Bugzilla resolution to transition the report into.
-                # DUPLICATE requires $dupeid to be passed in.
-            $dupeid
-                # Bugzilla report ID that this bug is being closed as
-                # duplicate of.
-                # Requires $new_resolution to be DUPLICATE.
-            $new_fixed_in
-                # OPTIONAL String representing version of product/component
-                # that bug is fixed in.
-            $comment
-                # OPTIONAL Text string containing comment to add.
-            $isprivate
-                # OPTIONAL Whether the comment will be private to the
-                # 'private_comment' Bugzilla group.
-                # Default: false
-            $private_in_it
-                # OPTIONAL if true will make the comment private in
-                # Issue Tracker
-                # Default: follows $isprivate
-            $nomail
-                # OPTIONAL Flag that is either 1 or 0 if you want email to be sent or not for this change
-        '''
-        return self._proxy.bugzilla.closeBug(id,resolution,self.user,'',
-                dupeid,fixedin,comment,isprivate,private_in_it,nomail)
-    def _setassignee(self,id,**data):
-        '''Raw xmlrpc call to set one of the assignee fields on a bug.
-        changeAssignment($id, $data, $username, $password)
-        data: 'assigned_to','reporter','qa_contact','comment'
-        returns: [$id, $mailresults]'''
-        return self._proxy.bugzilla.changeAssignment(id,data)
-    def _updatedeps(self,id,blocked,dependson,action):
-        '''update the deps (blocked/dependson) for the given bug.
-        blocked/dependson: list of bug ids/aliases
-        action: 'add' or 'delete'
-
-        RHBZ call:
-        updateDepends($bug_id,$data,$username,$password,$nodependencyemail)
-        #data: 'blocked'=>id,'dependson'=>id,'action' => ('add','remove')
-
-        RHBZ only does one bug at a time, so this method will loop through
-        the blocked/dependson lists. This may be slow.
-        '''
-        r = []
-        # Massage input to match what RHBZ expects
-        if action == 'delete':
-            action == 'remove'
-        data = {'id':id, 'action':action, 'blocked':'', 'dependson':''}
-        for b in blocked:
-            data['blocked'] = b
-            self._proxy.bugzilla.updateDepends(id,data)
-        data['blocked'] = ''
-        for d in dependson:
-            data['dependson'] = d
-            self._proxy.bugzilla.updateDepends(id,data)
-    def _updatecc(self,id,cclist,action,comment='',nomail=False):
-        '''Updates the CC list using the action and account list specified.
-        cclist must be a list (not a tuple!) of addresses.
-        action may be 'add', 'delete', or 'overwrite'.
-        comment specifies an optional comment to add to the bug.
-        if mail is True, email will be generated for this change.
-        '''
-        # Massage the 'action' param into what the old updateCC call expects
-        if action == 'delete':
-            action = 'remove'
-        elif action == 'overwrite':
-            action = 'makeexact'
-        data = {'id':id, 'action':action, 'cc':','.join(cclist),
-                'comment':comment, 'nomail':nomail}
-        return self._proxy.bugzilla.updateCC(data)
-    def _updatewhiteboard(self,id,text,which,action,comment,private):
-        '''Update the whiteboard given by 'which' for the given bug.
-        performs the given action (which may be 'append',' prepend', or
-        'overwrite') using the given text.'''
-        data = {'type':which,'text':text,'action':action}
-        if comment is not None:
-            data['comment'] = comment
-            if private:
-                data['commentprivacy'] = True
-        return self._proxy.bugzilla.updateWhiteboard(id,data)
     # TODO: update this when the XMLRPC interface grows requestee support
     def _updateflags(self,id,flags):
         '''Updates the flags associated with a bug report.
@@ -324,12 +142,6 @@ class RHBugzilla(bugzilla.base.BugzillaBase):
 
     #---- createbug - call to create a new bug
 
-    def _createbug(self,**data):
-        '''Raw xmlrpc call for createBug() Doesn't bother guessing defaults
-        or checking argument validity. Use with care.
-        Returns bug_id'''
-        r = self._proxy.bugzilla.createBug(data)
-        return r[0]
     # Methods for updating a user
     def _updateperms(self,user,action,groups):
         r = self._proxy.bugzilla.updatePerms(user, action, groups, self.user,
@@ -357,34 +169,6 @@ class RHBugzilla(bugzilla.base.BugzillaBase):
         r = self._proxy.bugzilla.editComponent(data,self.user,self.password)
         return r
 
-class RHBugzilla3(Bugzilla34, RHBugzilla):
-    '''Concrete implementation of the Bugzilla protocol. This one uses the
-    methods provided by Red Hat's Bugzilla 3.4+ instance, which is a superset
-    of the Bugzilla 3.4 methods. The additional methods (e.g. Bug.update)
-    should make their way into a later upstream Bugzilla release.
-
-    Note that RHBZ3 *also* supports most of the old RHBZ methods, under the
-    'bugzilla' namespace, so we use those when BZ3 methods aren't available.
-
-    This class was written using bugzilla.redhat.com's API docs:
-    https://bugzilla.redhat.com/docs/en/html/api/
-
-    By default, _getbugs will multicall getBug(id) multiple times, rather than
-    doing a single Bug.get(idlist) call. You can disable this behavior by
-    setting the 'multicall' property to False. This will make it somewhat
-    faster, but any missing/unreadable bugs will cause the entire call to
-    Fault rather than returning any data.
-    '''
-
-    version = '0.2'
-    user_agent = bugzilla.base.user_agent + ' RHBugzilla3/%s' % version
-
-    def __init__(self,**kwargs):
-        super(RHBugzilla3, self).__init__(**kwargs)
-        self.user_agent = self.__class__.user_agent
-        self.multicall = kwargs.get('multicall',True)
-
-    # XXX it'd be nice if this wasn't just a copy of RHBugzilla's _getbugs
     def _getbugs(self,idlist):
         r = []
         if self.multicall:
@@ -399,30 +183,66 @@ class RHBugzilla3(Bugzilla34, RHBugzilla):
             r = bugzilla.base.replace_getbug_errors_with_None(raw_results)
         else:
             raw_results = self._proxy.Bug.get({'ids':idlist})
-            r = [i['internals'] for i in raw_results['bugs']]
+            r = [i for i in raw_results['bugs']]
         return r
 
-    # This can be removed once RHBZ supports BZ3.6's Bug.fields() method
-    _getbugfields = RHBugzilla._getbugfields
-    # Use the upstream versions of these methods rather than the RHBZ ones
-    _query = Bugzilla34._query
     # This can be activated once Bug.get() returns all the data that
     # RHBZ's getBug() does.
     #_getbugs = Bugzilla3._getbugs # Also _getbug, _getbugsimple, etc.
 
     #---- Methods for updating bugs.
 
+    def _add_bug_comment(self,ids,comment,is_private):
+        '''Add a new comment to a specified bug ID(s). Returns the comment
+        ID(s) array.
+        '''
+
+        ret = list()
+        for id in ids:
+            r = self._proxy.Bug.add_comment({'id': id, 'comment': comment,
+                                             'is_private': is_private})
+            if 'id' in r:
+                ret.append(r['id'])
+
+        return ret
+
     def _update_bugs(self,ids,updates):
         '''Update the given fields with the given data in one or more bugs.
         ids should be a list of integers or strings, representing bug ids or
         aliases.
         updates is a dict containing pairs like so: {'fieldname':'newvalue'}
         '''
-        # TODO document changeable fields & return values
-        # TODO I think we need to catch XMLRPC exceptions to get a useful
-        # return value
-        # NOTE: this will be moved to upstream Bugzilla someday
-        return self._proxy.Bug.update({'ids':ids,'updates':updates})
+        tmp = dict()
+        for key, value in updates.items():
+            if key != 'comment' and key != 'commentprivacy':
+                if key != 'fixed_in':
+                    tmp[key] = value
+                else:
+                    tmp['cf_' + key] = value
+
+        tmp['ids'] = ids
+        ret = self._proxy.Bug.update(tmp)
+
+        # Comments must be handled separately using add_comment() API, trying
+        # to update using update() API causes error 117 (Invalid Comment ID)
+        # For more information please refer to:
+        #   http://www.bugzilla.org/docs/4.(0|2)/en/html/api/Bugzilla/WebService/Bug.html
+        if 'comment' in updates:
+            comment = dict()
+            body = updates['comment']
+            if 'commentprivacy' in updates:
+                is_private = updates['commentprivacy']
+            else:
+                is_private = False
+
+            self._add_bug_comment(ids, body, is_private)
+
+        return ret
+
+    def _close_bug(id, updates):
+        updates['id'] = id
+
+        return self._proxy.Bug.closeBug(updates)
 
     def _update_bug(self,id,updates):
         '''Update a single bug, specified by integer ID or (string) bug alias.
@@ -438,7 +258,7 @@ class RHBugzilla3(Bugzilla34, RHBugzilla):
 
     def _setstatus(self,id,status,comment='',private=False,private_in_it=False,nomail=False):
         '''Set the status of the bug with the given ID.'''
-        update={'bug_status':status}
+        update={'status':status}
         if comment:
             update['comment'] = comment
             update['commentprivacy'] = private
@@ -457,8 +277,9 @@ class RHBugzilla3(Bugzilla34, RHBugzilla):
         if comment:
             update['comment'] = comment
             if isprivate:
-                update['commentprivacy'] = True
-        return self._update_bug(id,update)
+                update['comment_is_private'] = True
+        return self._close_bug(id, update)
+        #return self._update_bug(id,update)
 
     def _setassignee(self,id,**data):
         '''Raw xmlrpc call to set one of the assignee fields on a bug.
@@ -492,7 +313,13 @@ class RHBugzilla3(Bugzilla34, RHBugzilla):
             update['comment'] = comment
 
         if action in ('add','delete'):
-            update['%s_cc' % action] = cclist
+            # Action 'delete' has been changed to 'remove' in Bugzilla 4.0+
+            if action == 'delete':
+                action = 'remove'
+
+            update = dict()
+            update['cc'] = dict()
+            update['cc'][action] = cclist
             self._update_bug(id,update)
         elif action == 'overwrite':
             r = self._getbug(id)
@@ -533,3 +360,232 @@ class RHBugzilla3(Bugzilla34, RHBugzilla):
             if private:
                 update['commentprivacy'] = True
         self._update_bug(id,update)
+
+    def _getbugfields(self):
+        '''Get the list of valid fields for Bug objects'''
+        # This should be changed once upstream Bug.fields()
+        # doesn't take 50-60s(!)
+        r = self._proxy.bugzilla.getBugFields()
+        r.append('blockedby')
+        r.append('components')
+
+        # NOTE: the RHBZ version lists 'comments' and 'groups', and strips
+        # the 'cf_' from the beginning of custom fields.
+
+        # XXX - bugzilla lists for us "bug_status" which is wrong. making sure
+        #       we have the correct list
+        if 'bug_status' in r:
+            r.remove('bug_status')
+        if 'status' not in r:
+            r.append('status')
+
+        return r
+
+    #################
+    # Query methods #
+    #################
+
+    def pre_translation(self, query):
+        '''Translates the query for possible aliases'''
+        if 'bug_id' in query:
+            if type(query['bug_id']) is not list:
+                query['id'] = query['bug_id'].split(',')
+            else:
+                query['id'] = query['bug_id']
+            del query['bug_id']
+
+        if 'component' in query:
+            if type(query['component']) is not list:
+                query['component'] = query['component'].split(',')
+
+        if 'include_fields' not in query and 'column_list' not in query:
+            return
+
+        if 'include_fields' not in query:
+            query['include_fields'] = list()
+            if 'column_list' in query:
+                query['include_fields'] = query['column_list']
+                del query['column_list']
+
+        include_fields = query['include_fields']
+        for newname, oldname in self.field_aliases:
+            if oldname in include_fields:
+                include_fields.remove(oldname)
+                if newname not in include_fields:
+                    include_fields.append(newname)
+
+    def post_translation(self, query, bug):
+        '''Translates the query result'''
+        tmpstr = []
+        if 'flags' in bug:
+            for tmp in bug['flags']:
+                tmpstr.append("%s%s" % (tmp['name'], tmp['status']))
+
+            bug['flags'] = ", ".join(tmpstr)
+        if 'blocks' in bug:
+            if len(bug['blocks']) > 0:
+                bug['blockedby'] = ', '.join(map(str, bug['blocks']))
+                bug['blocked'] = ', '.join(map(str, bug['blocks']))
+            else:
+                bug['blockedby'] = ''
+                bug['blocked'] = ''
+        if 'keywords' in bug:
+            if len(bug['keywords']) > 0:
+                bug['keywords'] = ', '.join(bug['keywords'])
+            else:
+                bug['keywords'] = ''
+        if 'component' in bug:
+            # we have to emulate the old behavior and add 'components' as
+            # list instead
+            bug['components'] = bug['component']
+            bug['component'] = bug['component'][0]
+        if 'alias' in bug:
+            if len(bug['alias']) > 0:
+                bug['alias'] = ', '.join(bug['alias'])
+            else:
+                bug['alias'] = ''
+        if 'groups' in bug:
+            # groups went to the opposite direction: it got simpler
+            # instead of having name, ison, description, it's now just
+            # an array of strings of the groups the bug belongs to
+            # we're emulating the old behaviour here
+            tmp = list()
+            for g in bug['groups']:
+                t = {}
+                t['name'] = g
+                t['description'] = g
+                t['ison'] = 1
+                tmp.append(t)
+            bug['groups'] = tmp
+
+    def build_query(self, **kwargs):
+        query = {}
+
+        def add_email(key, count):
+            if not key in kwargs:
+                return count
+
+            value = kwargs.get(key)
+            del(kwargs[key])
+            if value is None:
+                return count
+
+            query["query_format"] = "advanced"
+            query['email%i' % count] = value
+            query['email%s%i' % (key, count)] = True
+            query['emailtype%i' % count] = kwargs.get("emailtype", "substring")
+            return count + 1
+
+        def add_boolean(kwkey, key, bool_id):
+            if not kwkey in kwargs:
+                return bool_id
+
+            value = kwargs.get(kwkey)
+            del(kwargs[kwkey])
+            if value is None:
+                return bool_id
+
+            query["query_format"] = "advanced"
+            for boolval in value:
+                and_count = 0
+                or_count = 0
+
+                def make_bool_str(prefix):
+                    return "%s%i-%i-%i" % (prefix, bool_id,
+                                           and_count, or_count)
+
+                for par in boolval.split(' '):
+                    field = None
+                    fval = par
+                    typ = kwargs.get("booleantype", "substring")
+
+                    if par.find('&') != -1:
+                        and_count += 1
+                    elif par.find('|') != -1:
+                        or_count += 1
+                    elif par.find('!') != -1:
+                         query['negate%i' % bool_id] = 1
+                    elif not key:
+                        if par.find('-') == -1:
+                            raise RuntimeError('Malformed boolean query: %s' %
+                                               value)
+
+                        args = par.split('-', 2)
+                        field = args[0]
+                        typ = args[1]
+                        fval = None
+                        if len(args) == 3:
+                            fval = args[2]
+                    else:
+                        field = key
+
+                    query[make_bool_str("field")] = field
+                    if fval:
+                        query[make_bool_str("value")] = fval
+                    query[make_bool_str("type")] = typ
+
+                bool_id += 1
+            return bool_id
+
+        # Use fancy email specification for RH bugzilla. It isn't
+        # strictly required, but is more powerful, and it is what
+        # bin/bugzilla historically generated. This requires
+        # query_format='advanced' which is an RHBZ only XMLRPC extension
+        email_count = 1
+        email_count = add_email("cc", email_count)
+        email_count = add_email("assigned_to", email_count)
+        email_count = add_email("reporter", email_count)
+        email_count = add_email("qa_contact", email_count)
+
+        chart_id = 0
+        chart_id = add_boolean("fixed_in", "fixed_in", chart_id)
+        chart_id = add_boolean("blocked", "blocked", chart_id)
+        chart_id = add_boolean("dependson", "dependson", chart_id)
+        chart_id = add_boolean("flag", "flagtypes.name", chart_id)
+        chart_id = add_boolean("qa_whiteboard", "cf_qa_whiteboard", chart_id)
+        chart_id = add_boolean("devel_whiteboard", "cf_devel_whiteboard",
+                               chart_id)
+        chart_id = add_boolean("alias", "alias", chart_id)
+        chart_id = add_boolean("boolean_query", None, chart_id)
+
+        newquery = Bugzilla4.build_query(self, **kwargs)
+        query.update(newquery)
+        self.pre_translation(query)
+        return query
+
+    def _query(self,query):
+        '''Query bugzilla and return a list of matching bugs.
+        query must be a dict with fields like those in in querydata['fields'].
+        You can also pass in keys called 'quicksearch' or 'savedsearch' -
+        'quicksearch' will do a quick keyword search like the simple search
+        on the Bugzilla home page.
+        'savedsearch' should be the name of a previously-saved search to
+        execute. You need to be logged in for this to work.
+        Returns a dict like this: {'bugs':buglist,
+                                   'sql':querystring}
+        buglist is a list of dicts describing bugs, and 'sql' contains the SQL
+        generated by executing the search.
+        You can also pass 'limit:[int]' to limit the number of results.
+        For more info, see:
+        http://www.bugzilla.org/docs/4.0/en/html/api/Bugzilla/WebService/Bug.html
+        '''
+        old = query.copy()
+        self.pre_translation(query)
+
+        if old != query:
+            log.debug("RHBugzilla altered query to: %s", query)
+
+        ret = self._proxy.Bug.search(query)
+
+        # Unfortunately we need a hack to preserve backwards
+        # compabibility with older RHBZ
+        for bug in ret['bugs']:
+            self.post_translation(query, bug)
+
+        return ret
+
+# Just for API back compat
+class RHBugzilla3(RHBugzilla):
+    pass
+class RHBugzilla4(RHBugzilla):
+    pass
